@@ -1,4 +1,7 @@
-"""Handling of objective functions and objective quantities."""
+"""
+A collection of objects and helper methods for defining objective functions
+used in topology optimization.
+"""
 import abc
 from collections import namedtuple
 from typing import Callable, List, Optional
@@ -17,9 +20,11 @@ class ObjectiveQuantity(abc.ABC):
     """A differentiable objective quantity.
 
     Attributes:
-        sim: the Meep simulation object with which the objective quantity is registered.
-        frequencies: the frequencies at which the objective quantity is evaluated.
-        num_freq: the number of frequencies at which the objective quantity is evaluated.
+        sim: the Meep simulation object used to register the objective quantity.
+        frequencies: the frequencies at which the objective quantity is
+            evaluated.
+        num_freq: the number of frequencies at which the objective quantity is
+            evaluated.
     """
 
     def __init__(self, sim):
@@ -53,7 +58,8 @@ class ObjectiveQuantity(abc.ABC):
             return self._eval
         else:
             raise RuntimeError(
-                "You must first run a forward simulation before requesting the evaluation of an objective quantity."
+                "You must first run a forward simulation before requesting the"
+                "evaluation of an objective quantity."
             )
 
     def _adj_src_scale(self, include_resolution=True):
@@ -90,15 +96,14 @@ class ObjectiveQuantity(abc.ABC):
             )
             * dt
             / np.sqrt(2 * np.pi)
-        )  # dtft
+        )
 
-        # Interestingly, the real parts of the DTFT and fourier transform match, but the imaginary parts are very different...
+        # Interestingly, the real parts of the DTFT and Fourier transform match,
+        # but the imaginary parts are very different...
         # fwd_dtft = src.fourier_transform(src.frequency)
-        """
-        For some reason, there seems to be an additional phase
-        factor at the center frequency that needs to be applied
-        to *all* frequencies...
-        """
+        #
+        # Note: for some reason, there seems to be an additional phase factor at
+        # the center frequency that needs to be applied to *all* frequencies...
         src_center_dtft = (
             np.matmul(
                 np.exp(
@@ -117,26 +122,29 @@ class ObjectiveQuantity(abc.ABC):
         adj_src_phase = np.exp(1j * np.angle(src_center_dtft)) * self.fwidth_scale
 
         if self._frequencies.size == 1:
-            # Single frequency simulations. We need to drive it with a time profile.
+            # Single-frequency simulations. Requires a time profile.
             scale = dV * iomega / fwd_dtft / adj_src_phase  # final scale factor
         else:
-            # multi frequency simulations
+            # Multi-frequency simulations.
             scale = dV * iomega / adj_src_phase
-        # compensate for the fact that real fields take the real part of the current,
-        # which halves the Fourier amplitude at the positive frequency (Re[J] = (J + J*)/2)
+
+        # Cmpensate for the fact that real fields take the real part of the
+        # current, which halves the Fourier amplitude at the positive frequency
+        # (i.e. Re[J] = (J + J*)/2).
         if self.sim.using_real_fields():
             scale *= 2
         return scale
 
     def _create_time_profile(self, fwidth_frac=0.1, adj_cutoff=5):
-        """Creates a time domain waveform for normalizing the adjoint source(s).
+        """Creates a time-domain waveform for normalizing the adjoint source(s).
 
-        For single frequency objective functions, we should generate a guassian pulse with a reasonable
-        bandwidth centered at said frequency.
+        For single-frequency objective functions, we should generate a Gaussian
+        pulse with a reasonable bandwidth centered at the given frequency.
 
         TODO:
-        The user may specify a scalar valued objective function across multiple frequencies (e.g. MSE) in
-        which case we should check that all the frequencies fit in the specified bandwidth.
+        The user may specify a scalar-valued objective function across multiple
+        frequencies (e.g. MSE) in which case we should check that all the
+        frequencies fit in the specified bandwidth.
         """
         self.fwidth_scale = np.exp(-2j * np.pi * adj_cutoff / fwidth_frac)
         return mp.GaussianSource(
@@ -147,23 +155,7 @@ class ObjectiveQuantity(abc.ABC):
 
 
 class EigenmodeCoefficient(ObjectiveQuantity):
-    """A frequency-dependent eigenmode coefficient.
-    Attributes:
-        volume: the volume over which the eigenmode coefficient is calculated.
-        mode: the eigenmode number.
-        forward: whether the forward or backward mode coefficient is returned as
-          the result of the evaluation.
-        kpoint_func: an optional k-point function to use when evaluating the eigenmode
-          coefficient. When specified, this overrides the effect of `forward`.
-        kpoint_func_overlap_idx: the index of the mode coefficient to return when
-          specifying `kpoint_func`. When specified, this overrides the effect of
-          `forward` and should have a value of either 0 or 1.
-        subtracted_dft_fields: the DFT fields obtained using `get_flux_data` from
-          a previous normalization run. This is subtracted from the DFT fields
-          of this mode monitor in order to improve the accuracy of the
-          reflectance measurement (i.e., the $S_{11}$ scattering parameter).
-          Default is None.
-    """
+    """A differentiable frequency-dependent eigenmode coefficient."""
 
     def __init__(
         self,
@@ -177,8 +169,32 @@ class EigenmodeCoefficient(ObjectiveQuantity):
         subtracted_dft_fields: Optional[FluxData] = None,
         **kwargs
     ):
-        """
-        + **`sim` [ `Simulation` ]** —
+        """Initialize an instance of a differentiable frequency-dependent
+        eigenmode coefficient.
+
+        Args:
+          sim: the Meep simulation object of the problem.
+          volume: the volume over which the eigenmode coefficient is calculated.
+          mode: the eigenmode number.
+          forward: whether the forward or backward mode coefficient is returned
+              as the result of the evaluation. Default is True.
+          kpoint_func: an optional k-point function to use when evaluating the
+              eigenmode coefficient. When specified, this overrides the effect
+              of `forward`.
+          kpoint_func_overlap_idx: the index of the mode coefficient to return
+              when specifying `kpoint_func`. When specified, this overrides the
+              effect of `forward` and should have a value of either 0 or 1.
+          decimation_factor: An integer used to specify the number of timesteps
+              between updates of the DFT fields. The default is 0, at which the
+              value is automatically determined from the Nyquist rate of the
+              bandwidth-limited sources and the DFT monitor. It can be turned
+              off by setting it to 1.
+          subtracted_dft_fields: the DFT fields obtained using `get_flux_data`
+              from a previous normalization run. This is subtracted from the
+              DFT fields of this mode monitor in order to improve the accuracy
+              of the reflectance measurement (i.e., the $S_{11}$ scattering
+              parameter). Default is None.
+          eigenmode_kwargs: additional keyword arguments for EigenModeSource.
         """
         super().__init__(sim)
         if kpoint_func_overlap_idx not in [0, 1]:
@@ -257,6 +273,12 @@ class EigenmodeCoefficient(ObjectiveQuantity):
         return [source]
 
     def __call__(self):
+        """The values of the eigenmode coefficient at each frequency.
+
+        Returns:
+            1D array of eigenmode coefficients for each frequency in
+            self.frequencies.
+        """
         if self.kpoint_func:
             kpoint_func = self.kpoint_func
             overlap_idx = self.kpoint_func_overlap_idx
@@ -284,16 +306,35 @@ class EigenmodeCoefficient(ObjectiveQuantity):
 
 
 class FourierFields(ObjectiveQuantity):
+    """A differentiable frequency-dependent Fourier fields (dft_fields)"""
+
     def __init__(
         self,
         sim: mp.Simulation,
         volume: mp.Volume,
-        component: List[int],
+        component: int,
         yee_grid: Optional[bool] = False,
         decimation_factor: Optional[int] = 0,
         subtracted_dft_fields: Optional[FluxData] = None,
     ):
-        """ """
+        """Initialize an instance of differentiable Fourier fields instance.
+
+        Args:
+          sim: the Meep simulation object of the problem.
+          volume: the volume over which the eigenmode coefficient is calculated. Due to an unresolved bug,
+            the size must not be zero in at least one direction.
+          component: field component (e.g. mp.Ex, mp.Hz, etc.) of the Fourier fields
+          yee_grid: if True, the Fourier fields are evaluated at the corresponding Yee grid points;
+            otherwise, they are interpolated fields at the center of each voxel. Default is False
+          decimation_factor: An integer used to specify the number of timesteps between updates of
+            the DFT fields. The default is 0, at which the value is automatically determined from the
+            Nyquist rate of the bandwidth-limited sources and the DFT monitor. It can be turned off
+            by setting it to 1.
+          subtracted_dft_fields: the DFT fields obtained using `get_flux_data` from
+            a previous normalization run. This is subtracted from the DFT fields
+            of this mode monitor in order to improve the accuracy of the
+            reflectance measurement (i.e., the $S_{11}$ scattering parameter). Default is None.
+        """
         super().__init__(sim)
         self.volume = sim._fit_volume_to_simulation(volume)
         self.component = component
@@ -344,11 +385,11 @@ class FourierFields(ObjectiveQuantity):
         scale factor for each point (rather than a scale vector).
         """
 
-        self.all_fouriersrcdata = self._monitor.swigobj.fourier_sourcedata(
+        all_fouriersrcdata = self._monitor.swigobj.fourier_sourcedata(
             self.volume.swigobj, self.component, self.sim.fields, dJ
         )
 
-        for fourier_data in self.all_fouriersrcdata:
+        for fourier_data in all_fouriersrcdata:
             amp_arr = np.array(fourier_data.amp_arr).reshape(-1, self.num_freq)
             scale = amp_arr * self._adj_src_scale(include_resolution=False)
 
@@ -375,6 +416,13 @@ class FourierFields(ObjectiveQuantity):
         return sources
 
     def __call__(self):
+        """The values of Fourier Fields at each frequency
+
+        Returns:
+            array of Fourier Fields with dimension k+1 where k is the dimension of self.volume
+            The first axis corresponds to the index of frequency, and the rest k axis are for
+            the spatial indices of points in the monitor
+        """
         self._eval = np.array(
             [
                 self.sim.get_dft_array(self._monitor, self.component, i)
@@ -385,27 +433,50 @@ class FourierFields(ObjectiveQuantity):
 
 
 class Near2FarFields(ObjectiveQuantity):
+    """A differentiable near2far field transformation"""
+
     def __init__(
         self,
         sim: mp.Simulation,
         Near2FarRegions: List[mp.Near2FarRegion],
         far_pts: List[mp.Vector3],
+        nperiods: Optional[int] = 1,
         decimation_factor: Optional[int] = 0,
         norm_near_fields: Optional[NearToFarData] = None,
     ):
-        """ """
+        """Initialize an instance of differentiable Fourier fields instance.
+
+        Args:
+          sim: the Meep simulation object of the problem.
+          Near2FarRegions: List of mp.Near2FarRegion over which the near fields are collected
+          far_pts: list of far points at which fields are computed
+          nperiods: If nperiods > 1, sum of 2*nperiods+1 Bloch-periodic copies of near fields
+            is computed to approximate the lattice sum from Bloch periodic boundary condition.
+            Default is 1 (no sum).
+          decimation_factor: An integer used to specify the number of timesteps between updates of
+            the DFT fields. The default is 0, at which the value is automatically determined from the
+            Nyquist rate of the bandwidth-limited sources and the DFT monitor. It can be turned off
+            by setting it to 1.
+          norm_near_fields: the DFT fields obtained using `get_near2far_data` from
+            a previous normalization run. This is subtracted from the DFT fields
+            of this near2far monitor in order to improve the accuracy of the
+            reflectance measurement (i.e., the $S_{11}$ scattering parameter).
+            Default is None.
+        """
         super().__init__(sim)
         self.Near2FarRegions = Near2FarRegions
         self.far_pts = far_pts  # list of far pts
         self._nfar_pts = len(far_pts)
         self.decimation_factor = decimation_factor
         self.norm_near_fields = norm_near_fields
+        self.nperiods = nperiods
 
     def register_monitors(self, frequencies):
         self._frequencies = np.asarray(frequencies)
         self._monitor = self.sim.add_near2far(
             self._frequencies,
             *self.Near2FarRegions,
+            nperiods=self.nperiods,
             decimation_factor=self.decimation_factor,
         )
         if self.norm_near_fields is not None:
@@ -458,6 +529,13 @@ class Near2FarFields(ObjectiveQuantity):
         return sources
 
     def __call__(self):
+        """The values of far fields at each points at each frequency
+
+        Returns:
+            3D array of far fields. The first axis is the index of far field points in self.far_pts;
+            the second axis is the index of frequency; and the third is the index of component in
+            [mp.Ex(mp.Er), mp.Ey(mp.Ep), mp.Ez, mp.Hx(mp.Hr), mp.Hy(mp.Hp), mp.Hz]
+        """
         self._eval = np.array(
             [self.sim.get_farfield(self._monitor, far_pt) for far_pt in self.far_pts]
         ).reshape((self._nfar_pts, self.num_freq, 6))
@@ -465,17 +543,20 @@ class Near2FarFields(ObjectiveQuantity):
 
 
 class LDOS(ObjectiveQuantity):
-    def __init__(
-        self, sim: mp.Simulation, decimation_factor: Optional[int] = 0, **kwargs
-    ):
-        """ """
+    """A differentiable LDOS"""
+
+    def __init__(self, sim: mp.Simulation, **kwargs):
+        """Initialize a differentiable LDOS instance
+
+        Args:
+          sim: the Meep simulation object of the problem.
+        """
         super().__init__(sim)
-        self.decimation_factor = decimation_factor
         self.srckwarg = kwargs
 
     def register_monitors(self, frequencies):
         self._frequencies = np.asarray(frequencies)
-        self.forward_src = self.sim.sources
+        self._forward_src = self.sim.sources
         return
 
     def place_adjoint_source(self, dJ):
@@ -485,7 +566,7 @@ class LDOS(ObjectiveQuantity):
         dJ = dJ.flatten()
         sources = []
         forward_f_scale = np.array(
-            [self.ldos_scale / self.ldos_Jdata[k] for k in range(self.num_freq)]
+            [self._ldos_scale / self._ldos_Jdata[k] for k in range(self.num_freq)]
         )
         if self._frequencies.size == 1:
             amp = (dJ * self._adj_src_scale(False) * forward_f_scale)[0]
@@ -499,7 +580,7 @@ class LDOS(ObjectiveQuantity):
                 self.sim.fields.dt,
             )
             amp = 1
-        for forward_src_i in self.forward_src:
+        for forward_src_i in self._forward_src:
             if isinstance(forward_src_i, mp.EigenModeSource):
                 src_i = mp.EigenModeSource(
                     src,
@@ -527,7 +608,12 @@ class LDOS(ObjectiveQuantity):
         return sources
 
     def __call__(self):
+        """The values of LDOS at each frequency
+
+        Returns:
+            1D array of LDOS corresponding to each of self.frequencies
+        """
         self._eval = self.sim.ldos_data
-        self.ldos_scale = self.sim.ldos_scale
-        self.ldos_Jdata = self.sim.ldos_Jdata
+        self._ldos_scale = self.sim.ldos_scale
+        self._ldos_Jdata = self.sim.ldos_Jdata
         return np.array(self._eval)
