@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2019 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2024 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,11 @@ void set_random_seed(unsigned long seed) {
   meep_mt_init_genrand(seed);
 }
 
+void restore_random_seed() {
+  init_rand();
+  meep_mt_restore_genrand();
+}
+
 int random_int(int a, int b) {
   init_rand();
   return a + meep_mt_genrand_int32() % (b - a + 1);
@@ -60,9 +65,7 @@ double gaussian_random(double mean, double stddev) {
     s = v1 * v1 + v2 * v2;
   } while (s >= 1.0);
   if (s == 0) { return mean; }
-  else {
-    return mean + v1 * sqrt(-2 * log(s) / s) * stddev;
-  }
+  else { return mean + v1 * sqrt(-2 * log(s) / s) * stddev; }
 }
 
 } // namespace meep
